import telebot
from telebot import types
import mysql.connector
import random
import datetime
import logging
import re
from urllib.parse import quote

# تنظیم لاگینگ
logging.basicConfig(filename='bot_log.txt', level=logging.ERROR,
                   format='%(asctime)s - %(levelname)s - %(message)s')

# توکن ربات اصلی
MAIN_BOT_TOKEN = "7326675348:AAFzVRVTiqIF6LqtIcAlhukbdWr5d1LdP-s"
bot = telebot.TeleBot(MAIN_BOT_TOKEN)

# تنظیمات دیتابیس
DB_CONFIG = {
    'host': 'localhost',
    'user': 'gtalkir_testbotvam',
    'password': ',J{hYQ6hsXG(',
    'database': 'gtalkir_testbotvam'
}

# دیکشنری‌های ذخیره موقت
user_dict = {}
user_step = {}

def connect_to_database():
    """برقراری اتصال به دیتابیس"""
    try:
        conn = mysql.connector.connect(**DB_CONFIG)
        return conn
    except mysql.connector.Error as err:
        logging.error(f"خطا در اتصال به دیتابیس: {err}")
        return None

def setup_database():
    """راه‌اندازی جداول دیتابیس"""
    conn = connect_to_database()
    if conn:
        try:
            cursor = conn.cursor()
            # جدول کاربران
            cursor.execute('''
                CREATE TABLE IF NOT EXISTS users (
                    user_id INT AUTO_INCREMENT PRIMARY KEY,
                    telegram_id BIGINT UNIQUE,
                    first_name VARCHAR(100),
                    last_name VARCHAR(100),
                    national_id VARCHAR(10) UNIQUE,
                    phone VARCHAR(11) UNIQUE,
                    birth_date VARCHAR(10),
                    tracking_code VARCHAR(8) UNIQUE,
                    verification_link1 TEXT,
                    verification_link2 TEXT,
                    verification_link3 TEXT,
                    status ENUM('pending', 'approved', 'rejected', 'needs_correction', 'blocked') DEFAULT 'pending',
                    admin_comment TEXT,
                    registration_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                    last_update TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
                ) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci
            ''')
            
            conn.commit()
            print("جداول با موفقیت ایجاد شدند")
        except mysql.connector.Error as err:
            logging.error(f"خطا در ایجاد جداول: {err}")
        finally:
            conn.close()

def generate_unique_tracking_code():
    """تولید کد پیگیری یکتا"""
    try:
        while True:
            code = ''.join(random.choices('0123456789', k=8))
            conn = connect_to_database()
            if conn:
                cursor = conn.cursor()
                cursor.execute("SELECT COUNT(*) FROM users WHERE tracking_code = %s", (code,))
                if cursor.fetchone()[0] == 0:
                    conn.close()
                    return code
                conn.close()
    except Exception as e:
        logging.error(f"خطا در تولید کد پیگیری: {e}")
        return None

def validate_national_id(national_id):
    """اعتبارسنجی کد ملی"""
    if not re.match(r'^\d{10}$', national_id):
        return False
    
    check = int(national_id[-1])
    temp = sum(int(national_id[i]) * (10 - i) for i in range(9)) % 11
    
    return (temp < 2 and check == temp) or (temp >= 2 and check + temp == 11)

def validate_phone(phone):
    """اعتبارسنجی شماره موبایل"""
    return bool(re.match(r'^09\d{9}$', phone))

def validate_date(date_str):
    """اعتبارسنجی تاریخ تولد"""
    try:
        day, month, year = map(int, date_str.split('/'))
        if not (1300 <= year <= 1402):
            return False
        datetime.date(year, month, day)
        return True
    except:
        return False

def validate_link(link):
    """اعتبارسنجی لینک"""
    return bool(re.match(r'^https?://(?:[\w-]+\.)+[\w-]+(?:/[\w\-\.\?\%\&\=]*)?$', link))

def check_existing_data(field_name, value):
    """بررسی وجود داده تکراری در دیتابیس"""
    try:
        conn = connect_to_database()
        if conn:
            cursor = conn.cursor()
            cursor.execute(f"SELECT COUNT(*) FROM users WHERE {field_name} = %s", (value,))
            count = cursor.fetchone()[0]
            conn.close()
            return count > 0
        return False
    except Exception as e:
        logging.error(f"خطا در بررسی داده تکراری: {e}")
        return False

def get_markup(back=True, home=False):
    """ایجاد دکمه‌های مناسب برای هر مرحله"""
    markup = types.ReplyKeyboardMarkup(resize_keyboard=True, row_width=2)
    buttons = []
    
    if back:
        buttons.append(types.KeyboardButton("↩️ برگشت"))
    if home:
        buttons.append(types.KeyboardButton("🏠 منوی اصلی"))
    
    markup.add(*buttons)
    return markup

@bot.message_handler(commands=['start'])
def start(message):
    try:
        telegram_id = message.from_user.id
        user_step[telegram_id] = 'start'
        
        conn = connect_to_database()
        if not conn:
            bot.reply_to(message, '❌ خطا در ارتباط با سرور. لطفا بعداً تلاش کنید.')
            return
            
        cursor = conn.cursor(dictionary=True)
        cursor.execute("SELECT status FROM users WHERE telegram_id = %s", (telegram_id,))
        user = cursor.fetchone()
        conn.close()

        if user:
            status = user['status']
            if status == 'blocked':
                bot.reply_to(message, '⛔️ دسترسی شما به ربات مسدود شده است.')
                return
                
            status_messages = {
                'pending': '⏳ درخواست شما در حال بررسی است.',
                'approved': '✅ درخواست شما تأیید شده است.',
                'rejected': '❌ درخواست شما رد شده است.',
                'needs_correction': '🔄 درخواست شما نیاز به اصلاح دارد.'
            }
            
            markup = types.ReplyKeyboardMarkup(resize_keyboard=True, row_width=1)
            if status == 'rejected':
                markup.add(types.KeyboardButton("📝 ثبت نام مجدد"))
            elif status == 'needs_correction':
                markup.add(types.KeyboardButton("✏️ ویرایش اطلاعات"))
                
            markup.add(types.KeyboardButton("🔍 پیگیری درخواست"))
            
            bot.reply_to(
                message, 
                f"👋 سلام!\n\n{status_messages.get(status, '❓ وضعیت نامشخص')}", 
                reply_markup=markup
            )
        else:
            markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
            markup.add(types.KeyboardButton("📝 شروع ثبت نام"))
            
            welcome_text = """
👋 سلام! به ربات خوش آمدید

📌 راهنمای ثبت نام:
- لطفا اطلاعات خود را با دقت وارد کنید
- در هر مرحله می‌توانید به مرحله قبل برگردید
- پس از تکمیل ثبت نام، درخواست شما بررسی خواهد شد

🔸 برای شروع ثبت نام، روی دکمه زیر کلیک کنید
"""
            bot.reply_to(message, welcome_text, reply_markup=markup)
            
    except Exception as e:
        logging.error(f"خطا در دستور start: {e}")
        bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

@bot.message_handler(func=lambda message: message.text == "🔍 پیگیری درخواست")
def check_status(message):
    """بررسی وضعیت درخواست"""
    try:
        telegram_id = message.from_user.id
        conn = connect_to_database()
        if not conn:
            bot.reply_to(message, '❌ خطا در ارتباط با سرور. لطفا بعداً تلاش کنید.')
            return

        cursor = conn.cursor(dictionary=True)
        cursor.execute("""
            SELECT tracking_code, status, admin_comment, registration_date 
            FROM users WHERE telegram_id = %s
        """, (telegram_id,))
        user = cursor.fetchone()
        conn.close()

        if not user:
            bot.reply_to(message, '❌ درخواستی برای شما ثبت نشده است.')
            return

        status_emoji = {
            'pending': '⏳',
            'approved': '✅',
            'rejected': '❌',
            'needs_correction': '🔄',
            'blocked': '🚫'
        }

        status_text = f"""
🎫 کد پیگیری: {user['tracking_code']}
📊 وضعیت: {status_emoji.get(user['status'], '❓')} {user['status']}
📅 تاریخ ثبت: {user['registration_date'].strftime('%Y/%m/%d %H:%M:%S')}
"""
        if user['admin_comment']:
            status_text += f"\n💬 توضیحات: {user['admin_comment']}"

        bot.reply_to(message, status_text)

    except Exception as e:
        logging.error(f"خطا در بررسی وضعیت: {e}")
        bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

@bot.message_handler(func=lambda message: message.text in ["📝 شروع ثبت نام", "📝 ثبت نام مجدد"])
def start_registration(message):
    try:
        telegram_id = message.from_user.id
        user_step[telegram_id] = 'firstname'
        user_dict[telegram_id] = {'telegram_id': telegram_id}
        
        markup = get_markup(back=False)
        msg = bot.reply_to(
            message,
            "👤 لطفا نام خود را وارد کنید:",
            reply_markup=markup
        )
        bot.register_next_step_handler(msg, process_firstname_step)
        
    except Exception as e:
        logging.error(f"خطا در شروع ثبت نام: {e}")
        bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_firstname_step(message):
   try:
       if message.text == "↩️ برگشت":
           start(message)
           return
           
       telegram_id = message.from_user.id
       name = message.text.strip()
       
       if len(name) < 2:
           msg = bot.reply_to(
               message,
               "❌ نام باید حداقل 2 حرف باشد.\n\n👤 لطفا نام صحیح را وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_firstname_step)
           return
           
       user_dict[telegram_id]['firstname'] = name
       user_step[telegram_id] = 'lastname'
       
       msg = bot.reply_to(
           message,
           "👥 لطفا نام خانوادگی خود را وارد کنید:",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_lastname_step)
       
   except Exception as e:
       logging.error(f"خطا در ثبت نام: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_lastname_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "👤 لطفا نام خود را وارد کنید:", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_firstname_step)
           return

       telegram_id = message.from_user.id
       lastname = message.text.strip()
       
       if len(lastname) < 2:
           msg = bot.reply_to(
               message,
               "❌ نام خانوادگی باید حداقل 2 حرف باشد.\n\n👥 لطفا نام خانوادگی صحیح را وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_lastname_step)
           return
           
       user_dict[telegram_id]['lastname'] = lastname
       user_step[telegram_id] = 'national_id'
       
       msg = bot.reply_to(
           message,
           "🆔 لطفا کد ملی خود را وارد کنید (10 رقم):",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_national_id_step)
       
   except Exception as e:
       logging.error(f"خطا در ثبت نام خانوادگی: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_national_id_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "👥 لطفا نام خانوادگی خود را وارد کنید:", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_lastname_step)
           return

       telegram_id = message.from_user.id
       national_id = message.text.strip()
       
       if not validate_national_id(national_id):
           msg = bot.reply_to(
               message,
               "❌ کد ملی نامعتبر است.\n\n🆔 لطفا یک کد ملی 10 رقمی معتبر وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_national_id_step)
           return
           
       if check_existing_data('national_id', national_id):
           msg = bot.reply_to(
               message,
               "❌ این کد ملی قبلاً ثبت شده است.\n\n🆔 لطفا کد ملی دیگری وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_national_id_step)
           return
           
       user_dict[telegram_id]['national_id'] = national_id
       user_step[telegram_id] = 'phone'
       
       msg = bot.reply_to(
           message,
           "📱 لطفا شماره موبایل خود را وارد کنید (مثال: 09123456789):",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_phone_step)
       
   except Exception as e:
       logging.error(f"خطا در ثبت کد ملی: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_phone_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "🆔 لطفا کد ملی خود را وارد کنید (10 رقم):", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_national_id_step)
           return

       telegram_id = message.from_user.id
       phone = message.text.strip()
       
       if not validate_phone(phone):
           msg = bot.reply_to(
               message,
               "❌ شماره موبایل نامعتبر است.\n\n📱 لطفا یک شماره موبایل معتبر وارد کنید (مثال: 09123456789):",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_phone_step)
           return
           
       if check_existing_data('phone', phone):
           msg = bot.reply_to(
               message,
               "❌ این شماره موبایل قبلاً ثبت شده است.\n\n📱 لطفا شماره دیگری وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_phone_step)
           return
           
       user_dict[telegram_id]['phone'] = phone
       user_step[telegram_id] = 'birth_date'
       
       msg = bot.reply_to(
           message,
           "📅 لطفا تاریخ تولد خود را وارد کنید (مثال: 15/06/1370):",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_birth_date_step)
       
   except Exception as e:
       logging.error(f"خطا در ثبت شماره موبایل: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_birth_date_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "📱 لطفا شماره موبایل خود را وارد کنید:", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_phone_step)
           return

       telegram_id = message.from_user.id
       birth_date = message.text.strip()
       
       if not validate_date(birth_date):
           msg = bot.reply_to(
               message,
               "❌ تاریخ نامعتبر است.\n\n📅 لطفا تاریخ تولد را با فرمت صحیح وارد کنید (مثال: 15/06/1370):",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_birth_date_step)
           return
           
       user_dict[telegram_id]['birth_date'] = birth_date
       user_step[telegram_id] = 'link1'
       
       msg = bot.reply_to(
           message,
           "🔗 لطفا لینک احراز هویت متقاضی را وارد کنید:",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_link1_step)
       
   except Exception as e:
       logging.error(f"خطا در ثبت تاریخ تولد: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_link1_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "📅 لطفا تاریخ تولد خود را وارد کنید (مثال: 15/06/1370):", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_birth_date_step)
           return

       telegram_id = message.from_user.id
       link1 = message.text.strip()

       if not validate_link(link1):
           msg = bot.reply_to(
               message,
               "❌ لینک نامعتبر است.\n\n🔗 لطفا یک لینک معتبر وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_link1_step)
           return

       user_dict[telegram_id]['link1'] = quote(link1)
       user_step[telegram_id] = 'link2'

       msg = bot.reply_to(
           message,
           "🔗 لطفا لینک احراز هویت ضامن اول را وارد کنید:",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_link2_step)

   except Exception as e:
       logging.error(f"خطا در ثبت لینک اول: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_link2_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "🔗 لطفا لینک احراز هویت متقاضی را وارد کنید:", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_link1_step)
           return

       telegram_id = message.from_user.id
       link2 = message.text.strip()

       if not validate_link(link2):
           msg = bot.reply_to(
               message,
               "❌ لینک نامعتبر است.\n\n🔗 لطفا یک لینک معتبر وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_link2_step)
           return

       user_dict[telegram_id]['link2'] = quote(link2)
       user_step[telegram_id] = 'link3'

       msg = bot.reply_to(
           message,
           "🔗 لطفا لینک احراز هویت ضامن دوم را وارد کنید:",
           reply_markup=get_markup()
       )
       bot.register_next_step_handler(msg, process_link3_step)

   except Exception as e:
       logging.error(f"خطا در ثبت لینک دوم: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def process_link3_step(message):
   try:
       if message.text == "↩️ برگشت":
           msg = bot.reply_to(message, "🔗 لطفا لینک احراز هویت ضامن اول را وارد کنید:", reply_markup=get_markup())
           bot.register_next_step_handler(msg, process_link2_step)
           return

       telegram_id = message.from_user.id
       link3 = message.text.strip()

       if not validate_link(link3):
           msg = bot.reply_to(
               message,
               "❌ لینک نامعتبر است.\n\n🔗 لطفا یک لینک معتبر وارد کنید:",
               reply_markup=get_markup()
           )
           bot.register_next_step_handler(msg, process_link3_step)
           return

       user_dict[telegram_id]['link3'] = quote(link3)

       # نمایش خلاصه اطلاعات و درخواست تایید
       show_summary_and_confirm(message)

   except Exception as e:
       logging.error(f"خطا در ثبت لینک سوم: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')
       
def show_summary_and_confirm(message):
   try:
       telegram_id = message.from_user.id
       user_data = user_dict[telegram_id]
       
       summary = f"""
📋 خلاصه اطلاعات ثبت شده:

👤 نام: {user_data['firstname']}
👥 نام خانوادگی: {user_data['lastname']}
🆔 کد ملی: {user_data['national_id']}
📱 شماره موبایل: {user_data['phone']}
📅 تاریخ تولد: {user_data['birth_date']}

آیا اطلاعات وارد شده صحیح است؟
"""
       markup = types.ReplyKeyboardMarkup(resize_keyboard=True, row_width=2)
       markup.add(
           types.KeyboardButton("✅ تایید و ثبت نهایی"),
           types.KeyboardButton("❌ انصراف و شروع مجدد")
       )
       
       msg = bot.reply_to(message, summary, reply_markup=markup)
       bot.register_next_step_handler(msg, handle_final_confirmation)
       
   except Exception as e:
       logging.error(f"خطا در نمایش خلاصه اطلاعات: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def handle_final_confirmation(message):
   try:
       if message.text == "✅ تایید و ثبت نهایی":
           save_user_data(message)
       else:
           bot.reply_to(message, "❌ ثبت نام لغو شد. برای شروع مجدد /start را بزنید.")
           cleanup_user_data(message.from_user.id)
           
   except Exception as e:
       logging.error(f"خطا در تایید نهایی: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def save_user_data(message):
   try:
       telegram_id = message.from_user.id
       user_data = user_dict[telegram_id]
       
       tracking_code = generate_unique_tracking_code()
       if not tracking_code:
           bot.reply_to(message, '❌ خطا در تولید کد پیگیری. لطفا دوباره تلاش کنید.')
           return

       conn = connect_to_database()
       if not conn:
           bot.reply_to(message, '❌ خطا در ارتباط با سرور. لطفا بعداً تلاش کنید.')
           return
           
       cursor = conn.cursor()
       
       # ذخیره در دیتابیس
       sql = """INSERT INTO users 
               (telegram_id, first_name, last_name, national_id, phone, birth_date,
               tracking_code, verification_link1, verification_link2, verification_link3, status)
               VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, 'pending')"""
               
       values = (
           telegram_id,
           user_data['firstname'],
           user_data['lastname'],
           user_data['national_id'],
           user_data['phone'],
           user_data['birth_date'],
           tracking_code,
           user_data['link1'],
           user_data['link2'],
           user_data['link3']
       )
       
       cursor.execute(sql, values)
       conn.commit()
       
       success_message = f"""
✅ اطلاعات شما با موفقیت ثبت شد

🎫 کد پیگیری: {tracking_code}

⏳ درخواست شما در حال بررسی است
📨 نتیجه بررسی از طریق همین ربات به شما اعلام خواهد شد
"""
       markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
       markup.add(types.KeyboardButton("🔍 پیگیری درخواست"))
       
       bot.reply_to(message, success_message, reply_markup=markup)
       cleanup_user_data(telegram_id)
       
   except Exception as e:
       logging.error(f"خطا در ذخیره اطلاعات: {e}")
       bot.reply_to(message, '❌ خطایی رخ داد. لطفا دوباره تلاش کنید.')

def cleanup_user_data(telegram_id):
   """پاکسازی اطلاعات موقت کاربر"""
   if telegram_id in user_dict:
       del user_dict[telegram_id]
   if telegram_id in user_step:
       del user_step[telegram_id]

# شروع ربات
if __name__ == "__main__":
   try:
       setup_database()
       print("✅ ربات اصلی شروع به کار کرد...")
       bot.infinity_polling(timeout=10, long_polling_timeout=5)
   except Exception as e:
       logging.error(f"خطا در اجرای ربات: {e}")